eForth 68HC11 v93.02 Manual                              (c)93 W.Schemmert

Inhalt:
=======
1. Glossar, Kurzbeschreibung der Kernel-Worte
2. Tips zur Arbeitsweise mit eForth und FDOCK Terminalprogramm



1. Glossar:
===========
Das Glossar stammt aus "EFORTH.GLO" und wurde fr diese Version
aktualisiert (Leider gibts noch keine deutsche bersetzung)

WARNING: Advanced information -- subject to change.

================================================================

Sort order

!"#$%&'()*+,-./digits:;<=>?@Alpha[\]^_`{|}~

================================================================

Attributes  Capitalized symbols.

  C  the word may only be used during compilation of a colon definition.
  D  the word is a defining word.
  I  the word is IMMEDIATE and will execute during compilation, unless special action is taken.
  U  a user value.

================================================================

Stack notes

( compile \ run \ child ;Return ;Float ; <input stream> )

( before -- after ;R before -- after ;F before -- after ; <string> )

================================================================

Glossary

!   ( w a -- )    "store"
	Store a 16-bit number at aligned address.

!CSP    ( -- )  "set c s p"
	Save the values of the current stack pointers.

#   ( d -- d )  "number sign"
	Convert one digit of a number using the current base.
	Must be used within <# and #> .

#>  ( d -- b u )    "number sign greater"
	Terminate a numeric conversion.

#S  ( ud -- 0 0 )   "number sign s"
	Convert all digits of a number using the current base.

#TIB    ( -- a )   "number t i b"
	The system double variable which holds the size and aligned
	address of the terminal input buffer.

$"| ( -- $ )    C   "string quote primitive"
	Return the address of a compiled inline packed string.
	The run-time primitive compiled by " .

$," ( -- ; <string> )   "string comma quote"
	Compile an inline packed character string into the code area,
	terminated by the " double quote character.

$,n ( $ -- )    "string comma n"
	Create a name for a definition using string.
	Set the code pointer to the next free cell in the code area,
	no code is compiled.
	The name is not linked into the dictionary.

$COMPILE    ( $ -- )
	Convert a string to a word address.
	Execute the word in interpreting mode or compile it if in compiling mode.

$INTERPRET  ( $ -- )    "string interpret"
	At the interactive level, if a word is defined perform its action.
	If not, try to convert it to a number, if that fails, issue an error
	message.

'   ( -- ca ; <string> )    "tick"
	Return the code address of the word following.

'?KEY   ( -- a )    U   "tick question key"
	The system input device status vector.

'ECHO   ( -- a )    U   "tick echo"
	The system echo device vector.

'EMIT   ( -- a )    U   "tick emit"
	The system output device vector.

'EVAL   ( -- a )    "tick eval"
	The system interpret/compile vector.

'EXPECT ( -- a )    U   "tick expect"
	The system line input vector.

'NUMBER ( -- a )    U   "tick number"
	The system number conversion vector.

'PROMPT ( -- a )    U   "tick prompt"
	The system prompt vector.

'TAP    ( -- a )    U   "tick tap"
	The input case function vector.

(   ( -- ; <string> )   I   "paren"
	Begin a comment. The comment is terminated by the ) character.
	May be used inside or outside a definition.

*   ( n n -- n )    "star"
	Multiply two signed numbers. Return a 16-bit signed number.

*/  ( n1 n2 n3 -- q )   "star slash"
	Multiply n1 by n2 producing the 32-bit intermediate product d.
	Divide d by n3 producing a 16-bit quotient.

*/MOD   ( n1 n2 n3 -- r q ) "star slash mod"
	Multiply n1 by n2 producing the 32-bit intermediate product d.
	Divide d by n3 producing a 16-bit remainder and a 16-bit quotient.

+   ( w w -- w )    "plus"
	Addition.

+!  ( n a -- )  "plus store"
	Increment the 16-bit value at address by n.

,   ( w -- )    "comma"
	Compile a 16-bit value into the code area.

-   ( w w -- w )    "minus"
	Subtract the top from the second element on the data stack.

-TRAILING   ( b u -- b u )  "dash trailing"
	Adjust the count to eliminate any trailing white-space in the string.

.   ( n -- )    "dot"
	Display the single value, use the current base.
	If BASE is DECIMAL , display as a signed number.

."  ( -- ; <string> )   I,C "dot quote"
	Used only within a definition to compile an inline packed string
	terminated by the " double quote character.
	At run-time the string is displayed on the current output device.

."|  -- )   C   "dot quote primitive"
	Display a compiled inline packed string to the current output device.
	This run-time primitive is compiled by ." .

.(  ( -- ; <string> )   I   "dot paren"
	Begin a comment that is displayed to the current output device.
	May be used inside or outside a definition.

.ID ( na -- )   "dot i d"
	Display the packed string at address.

.OK ( -- )  "dot o k"
	Display the standard system prompt.

.S  ( ? -- ? )  "dot s"
	Display the contents of the data stack.

/   ( n n -- q )    "slash"
	Floored division for 16-bit numbers. Returns only the 16-bit quotient.

/MOD    ( n n -- r q )  "slash mod"
	Floored division for 16-bit numbers.
	16-bit remainder and 16-bit quotient.

0   ( -- n )
1   ( -- n )
-1  ( -- n )
	Primitives which put "0" rsp. "1"  "-1" on the data stack

0<  ( n -- t )  "zero less"
	Return true if n is less than 0, negative. Comparison is signed.
	Also used for sign extension.

0=  ( w -- t )  "zero equals"
	Return true if w is equal to 0.

1+  ( -- )
	Increases TOS by 1

1-  ( -- )
	Decreases TOS by 1


2!  ( d a -- )  "two store"
	Store a 32-bit value at aligned address.

2@  ( a -- d )  "two fetch"
	Return the 32-bit value stored at aligned address.

2DROP   ( d -- )    "two drop"
	Pop the 32-bit number, or the top two 16-bit numbers,
	from the data stack.

2DUP    ( d -- d d )    "two dupe"
	Duplicate the 32-bit number, or the top two 16-bit numbers,
	on the data stack.

:   ( -- ; <string> )   D   "colon"
	Begin a colon definition to be added to the current vocabulary.

;   ( -- )  I,C "semicolon"
	Terminate a colon definition begun with : .

<   ( n1 n2 -- t )  "less than"
	Return true if n1 is less than n2. Comparison is signed.

<#  ( -- )  "start number"  "less number sign"
	Begin a numeric conversion.

=   ( w w -- t )    "equals"
	Return true if w1 is equal to w2.

>CHAR   ( u -- c )  "to character"
	Convert a value to a printable character.
	Replace an unprintable character with the _ underscore character.

>IN ( -- a )    "to in"
	The pointer into the input stream.

>NAME   ( ca -- na, F ) "to name"
	If possible, convert a code address to a name address.
	If not possible, return a false flag.

>R  ( w -- ;R -- w )    C   "to r"
	Pop the top element of the data stack and Push it on the return stack.

?   ( a -- )    "question"
	Display the single value stored at address, use the current base.
	If BASE is DECIMAL , display as a signed number.

?branch ( f -- )    "question branch"
	Run time rouitne to redirect execution to the address in the next cell
	if flag is false.

?CSP    ( -- )  "question c s p"
	Compare the current value of the stack pointers with the saved values.
	ABORT with an error message if different.

?DUP    ( w -- w w, 0 ) "question dupe"
	Duplicate the number on top of the data stack only if it is non-zero.

 ?KEY  ( -- c T | F )   "question key"
	Return input character and true, or a false if no input.
	This is changed with respect to original eForth. The basic RX is
	now handled by an interrupt server, which does not appear in the
	Forth dictionary.

?STACK  ( -- )  "question stack"
	Display an error message if the stack limits have been exceeded.

?UNIQUE ( $ -- $ )  "question unique"
	Display a warning massage for a duplicate definition.

@   ( a -- w )  "fetch"
	Return the 16-bit value stored at aligned address.

@EXECUTE  ( a -- ) "fetch execute"
	Fetch the execution token stored at address and execute it, ie indirect
	execution. If the value contained in address is zero, do nothing.

ABORT   ( -- )
	Reset the data stack and perform the function of QUIT .
	Note, no message is displayed.

ABORT"  ( -- ; <string> \ f -- )    I,C abort quote"
	Used only within a definition to compile an inline packed string
	terminated the " double quote character. At run-time, if the flag
	is false, execute the sequence of words following the string.
	Otherwise, the string is displayed on the current output ce,
	execution is then passed to an error handling routine.

abort"  ( f -- )    C   "abort quote primitive"
	The run-time primitive compiled by ABORT" .

ABS     ( n -- +n )
	Return the absolute value of n.

accept  ( b u -- b u )
	Receive a line of u characters maximum to the an input buffer at byte
	address. Terminate input if a carriage return is received.
	Return the actual count of received characters.
	Use the current input device.

AFT     ( a -- a a \ -- )   I,C
	Used within a loop structure to unconditional skip a portion of code
	the first time thru the loop. AFT compiles the machine unconditional
	branch instruction and leaves an address to be resolved by THEN .

AGAIN   ( a -- \ -- )   I,C
	Terminate an infinite loop structure. AGAIN compiles an unconditional
	branch instruction, and uses the address left by BEGIN to resolve
	this backward branch.

AHEAD   ( -- a \ -- )   I,C
	Mark the beginning of a forward branching,unconditional branch structure. AHEAD compiles the machine unconditional branch instruction and leaves an address to be resolved by THEN .

ALIGNED ( b -- a )
	Convert a byte address to a word aligned address.

ALLOT   ( n -- )
	Adjust the code area pointer by n.

AND ( w w -- w )
	A bitwise logical AND.

AUTO  ( -- )
	Check conditions and, if desired, autocopy selected memory areas
	and run autostart application.

BASE    ( -- a )    U
	The system variable which holds the current numeric conversion radix.

BEGIN   ( -- a \ -- )   I,C
	Mark the beginning of an indefinite loop structure.
	Leave an address to be resolved by UNTIL, or WHILE and REPEAT .

BL  ( -- c ) "bl"
	Push the value of a space, the blank character, on the data stack.

branch  ( -- )  C
	Run time routine to redirect execution to the address in the next cell.

BREAK. ( -- )  "breakpoint" or "breakdot" whatever you prefer
	Shows two top items of Return Stack and five top items of
	Data Stack. Tries to reconstruct the name of the previously executed
	word. If fails, previous dictionary cell (before break.) is EMITed.
	Program execution continues after any input from terminal.
	No restrictions in use, interpreter mode or compiled (performed at
	runtime) .

BYE ( -- )
	Exit Forth and return to the underlying environment or DOS.

C,    ( b -- )
	compile one byte into the dictionary.

C!  ( v b -- )  "c store"
	Store an byte value at byte address.

C@  ( b -- v )  "c fetch"
	Return the byte value stored at byte address.

CALL,   ( a -- )    C   "call comma"
	Assemble a 4 byte subroutine call to the designated address.

CATCH   ( ca -- err#/0 )
	Setup a local error frame and execute the word referenced by the
	execution token ca. Return a non-zero error number or zero for no error.

CELL+   ( a1 -- a2 )    "cell plus"
	Add cell size in bytes to address a1.

CELL-   ( a1 -- a2 )    "cell minus"
	Subtract cell size in bytes to address a1.

CELLS   ( n1 -- n2 )
	Multiply n1 by the cell size in byte.

CHARS   ( +n c -- )
	Display +n of character to the current output device.

CLSL  ( -- )
	Logical Shift Left low-order byte of TOS by 1 bit position
	If the shiftet-out bit =1 : High-order byte of TOS = $01
	If the shiftet-out bit =0 : High-order byte of TOS = $00

CLSR  ( -- )
	Logical Shift Right low-order byte of TOS by 1 bit position
	If the shiftet-out bit =1 : High-order byte of TOS = $01
	If the shiftet-out bit =0 : High-order byte of TOS = $00

CMOVE   ( b1 b2 u -- )  "c move"
	Move u byte values from byte address b1 to b2, proceeding from lower
	to higher memory. Overwrite occurs if b1<b2<b1+u .

COLD    ( -- )
	Completely re-initialize the system.

COMPILE ( -- )  C
	Used only within a definition. At run-time the word following COMPILE
	is not executed, but its code address is copied into the code area.

COMPILEONLY ( -- )
	Make the last compiled word a compile only word.

CONSTANT ( w --  ; <string>)      D
	Compile a new constant initialized with value w

CONTEXT ( -- a )
	The variable used to specify the dictionary search order.

COUNT   ( $ -- b +n )
	Return the byte address and byte count of a packed string.

CR  ( -- )   carriage return"
	Position the cursor at the beginning of the next line of the current
	output device.

CREATE  ( -- ; <string> \ -- a )    D
	Build a named definition. At run-time the address pointing to next
	availabe code space is pushed on the data stack.

CSP ( -- a )    "c s p"
	The system variable which holds the current stack pointer.
	Used for error checking.

CURRENT ( -- a )
	The variable used to specify the vocabulary in which new definitions
	are compiled.

DECIMAL ( -- )
	Set decimal as the current BASE . Base 10.

DEPTH   ( -- n )
	The number of elements on the data stack, does not include n .

DIGIT   ( u -- c )
	Convert a single digit number to its character value.

DIGIT?  ( c base -- v t )   "digit question"
	Try to convert a character to binary digit.
	Return true if the digit is valid for the current base.

dm+ ( a u -- p a+ ) "d m plus"
	Display the 16-bit values starting at the aligned address a.

DNEGATE ( d -- -d ) "d negate"
	Return the two's complement a double number.
	Change the sign of a double number.

DO  ( u -- )    I,C
	Begin a down-counting loop.
	Repeat the loop till NEXT u+1 times from u to 0.

do$ ( -- $ )    C   "do string"
	Return the address of a compiled inline packed string.

doLIST  ( a -- )    C   "do list"
	The run-time routine which executes the list in a colon definition
	pointed to by a.

doLIT   ( -- n )    C   "do literal"
	Return the in-line literal compiled by LITERAL.

doLOOP  ( -- ) C
	Run-time routine to terminate a down-counting DO-ZLOOP loop. See ZLOOP.

doUSER  ( -- )  C   "do user variable"
	The run-time action of user variables.

doVAR( -- a )  C   "do variable"
	The run-time action of variable.

DP  ( -- a ) "d p"
	The pointer to the next available dictionary location in code space.
	Differing from the original eForth model, the dictionary here is
	organized like in conventional Forth systems, with header and code
	segments interleaved.

DROP    ( w -- )
	Remove the top element on the data stack.

DUMP    ( a u -- )
	Display the HEX and character values starting at aligned address a,
	for count u.

DUP ( w -- w w )    "dupe"
	Duplicate the top element on the data stack.

EEP!  ( w a -- )
	Pop the data stack to 68HC11 internal EEPROM

EEPBURN ( c b --)
	Hardware handler to burn one byte into 68HC11 internal EEPROM.
	To be called by "EEP!"

ELSE    ( -- \ a -- a ) I,C
	Used within a conditional branch structure.
	ELSE resolves a forward conditional branch compiled by IF .
	ELSE then compiles an unconditional branch instruction,
	leaving an address to be resolve by THEN .

EMIT    ( w -- )
	Output a character to the current output device.

END-HCODE  ( -- )
	compile "@NEXT" to dictionary and finish a primitive body

EVAL    ( -- )
	Interpret or compile the tokens from the input stream.

EXECUTE ( w -- ;R -- w )
	Execute the word definition indicated by the execution token w.

EXIT    ( -- ;R w -- )
	Compile a subroutine return.

EXTERNAL ( a -- ; <string> )     D
	Compile a call for external function.
	The executable machine code must begin at address a.
	Back to eForth at the first unnested "rts" instruction found.
	Free use of 68HC11 internal registers: A,B,X,Y and
	hardware stack, if left with equal stackpointer.
	Free use of memory, subroutines and interrupts as long as not
	destroing eForth setup.

FILE    ( -- )
	Specify system input from a file using pace handshake. File input
	is not echoed, all output messages are displayed.

FILL    ( b u v -- )
	Fill an area at byte address of length u using the byte value v.

find    ( $ va -- ca f, $ F )   "find primitive"
	Given a string and a dictionary entry thread, search for a name match.
	If found, return the code address and a true flag.
	If not found, the string address and a false flag.

FLIP         ( -- )
	   Swap both bytes of the TOS

FORGET   ( --; <string> )
	Erase the word with name <string> and all following from dictionary
	Reset current dictionary pointer to last remaining word.
	More sophisticated dictionary handling not yet implemented.

FORTH   ( -- )
	Make the default system vocabulary FORTH the context vocabulary.

frth ( -- a )     U
	User Variable: Point to the last name in the name dictionary.

HAND    ( -- )
	Specify system input from keyboard, no handshake. All input is echoed.

HANDLER ( -- a )
	The current error handler frame pointer.

HCODE  ( -- ;<string> )
	compile a primitive (machine HexCODE) header
	for example, enter:  HCODE EX $7E C, ' COLD , END-HCODE  and run EX

HERE    ( -- a )
	Push the address of the next free cell in the code area on the datastack.

HEX ( -- )
	Set hexadecimal as the current BASE . Base 16.

HI  ( -- )
	Display the sign-on message.

HLD ( -- a )    "h l d"
	The pointer to a formatted numeric output string.

HOLD    ( c -- )
	Insert the character in the formatted numeric output string.

IF  ( -- a \ f -- ) I,C
	Mark the beginning of a forward branching, conditional branch structure.
	IF compiles the machine conditional branch instruction and leaves
	an address to be resolved by THEN or ELSE .

IMMEDIATE   ( -- )
	Mark the most recently created dictionary entry as a word which
	will execute during compilation.

INVERT  ( w -- w )
	Bitwise logical invert. Equivalent to -1 XOR. The one's complement.

JOIN   ( wHI wLO -- w )
	Joins the lower bytes of the two stack top items
	to one word held in TOS

KEY ( -- c )
	Return a character from the current input device.
	If no key is ready, wait until one is available.

kTAP    ( b b b c -- b b b )    "k tap"
	The 'tap routine used for file input.

LAST    ( -- a )
	The pointer to the name of the most recently created dictionary entry.

LITERAL ( w -- \ -- w )I
	Compile a number as an inline value.

LOADBIN  ( a u -- )
	Loads u arbitrary bytes to address a via SerialCom.
	No semantic or allocation precautions, use only if you
	are looking through.

LOADSYSTEM  ( -- )
	Load a precompiled dictionary extension from the terminal.
	USER variables are refreshed, too.
	Data format see "SAVESYS"
	During loading, RAMBOT and UPP are compared with present
	values. If inadequate, loading is refused. ( Echo ESC)
	If OK, further loading is triggered by echoing PACE
	LAST is inherently restored by loading the user area.
	VARAD is temporarily saved before loading. If the previous
	value of VARAD is bigger than the loaded one, system VARAD is
	restored to the user area, assuming other users have defined
	additional public variables.
	THE VALUES OF PUBLIC VARIABLES ARE NOT CHANGED BY LOADSYS ! So,
	take care to reinitialize them by a RUNNING WORD after reloading
	the system. Simple ! commands are NOT PERFORMED at reload. A tricky
	opportunity to make errors, perhaps public variables should be
	saved too.

MS       ( u -- )
	Produce a time-delay of approx. u milliseconds with 8MHz quartz.
	For u=0, delay is 16 microseconds.
	Attention, when using different quartz frequencies !!

M*  ( n n -- d )    "m star"
	Multiply two signed numbers. Return a 32-bit signed number.

M/MOD   ( d n -- r q )  "m slash mod"
	Floored division of a 32-bit number divided by a 16-bit number.
	Return a 16-bit quotient and a 16-bit remainder.

MAX ( n n -- n )
	Leave the greater of the two signed values.

MIN ( n n -- n )
	Leave the smaller of the two signed values.

MOD ( n n -- r )
	Floored division for 16-bit numbers. Returns only the 16-bit remainder.

NAME?   ( $ -- ca f, $ F )  "name question"
	Given a string, search for a name match. If found, return the
	code address and a true flag. If not found, the string address and a
	false flag.

NEGATE  ( n -- -n )
	Equivalent to 0 SWAP-. The two's complement of a number.
	Change the sign of a number.

NUF?    ( -- t )    "nuf question"
	Continue until paused or terminated by user. Any key will pause,
	while paused any key except 'enter' will restart and return false,
	enter will return true.

NULL$   ( -- $ )    "null string"
	The address of a string with a zero count.

NUMBER? ( $ -- d T, $ F )   "number question"
	Try to convert a packed string to binary number.
	If possible return the number and a true.
	Otherwise, return the string address and false.
	A leading '$' for hexidecimal, a leading '-' for negative, and/or the
	decimal point for a double number. Accepts lower case HEX.

OR  ( w w -- w )
	A bitwise logical OR.

OVER    ( w1 w2 -- w1 w2 w1 )
	Copy the second element to the top of the data stack.

OVERT   ( -- )
	Used by ; to link a successfully defined word into the search order.

PACE    ( -- )
	Send the file transfer handshake character.

PACK$   ( b u $ -- $ )  "pack string"
	Move and convert the string at byte address with byte count
	to a packed string at address $ .

PAD     -- a )
	Short for scratch pad. Address of a temporary buffer.

PARSE   ( c -- b u ; <string> )
	Scan the current input stream for the given character as a delimiter.
	Return the beginning byte address and count of the delimited string.

pars    ( b u c -- b u delta ; <string> )
	scan string for the given character as a delimiter.
	Return the beginning byte address and count of the delimited string.
	Delta is the beginning to current offset.

PICK    ( +n -- w )
	Copy the +nth data stack value to the top of the data stack.

PRESET  ( -- )  C
	Clear the data stack, the return stack and initialize system.

QUERY   ( -- )
	Receive a line to the current input buffer from current input device.

QUIT    ( -- )
	Clear the return stack, set interpret state, and return control
	to the current command line interpreter.

R>  ( -- w ;R w -- )C  "r from"
	Pop the top element of the return stack and Push it on the data stack.

R@  ( -- w ;R w -- w )  "r fetch"
	Copy the top element of the return stack and Push it on the data stack.

REPEAT  ( a a -- \ -- ) I,C
	Terminate an indefinite loop structure. REPEAT compiles an
	unconditional branch instruction, and uses the address left
	by WHILE to resolve this backward branch.

ROT ( w1 w2 w3 -- w2 w3 w1 )    "rote"
	Rotate the top three elements on the data stack.
	Third element to top and all other shifted down.

RP! ( a -- )C      "r p store"
	Set the return stack pointer to address.

RP0 ( -- a )     "r p zero"
	Return the bottom address of the return stack pointer.

RP@ ( -- a      "r p fetch"
	Return the address of the return stack pointer.

SAME?   ( a1 a2 u -- a1' f \ -0+ )  "same question"
	Compare the two strings, return the beginning address of the
	first string and a truth flag.

SAVEBIN  ( a u -- )
	pushes u arbitrary bytes starting from address a
	via Serial Com. Uses XON/XOFF protocol. Simple data
	format without header nor security.

SAVESYSTEM  ( -- )
	save the dictionary extension beeing actually in the
	RAM area via SerialCom. Present protocol: binary byte
	stream without security precautions but using XON/XOFF protocol.
	Length counting is provided inherently. Total length of file
	is:   (DP-RAMBOT) + USOFF + 8  bytes
	Data format: (WORDs are transmitted INTEL-style LOfirst-HInext)
		   .WORD   RAMBOT
		   .WORD   UPP
		   .WORD   DP (last byte used by dictionary extension)
		   .WORD   USOFF
		   .BYTEs   dictionary extension from RAMBOT to DP
		   .BYTEs   user area from UPP to (UPP+USOFF)
	Attention: information about LAST and VARAD is contained
	in the user block!  BUT THIS WORD DOES NOT SAVE PUBLIC
	VARIABLES ! So, be aware itialize all VARIABLEs in your application
	by RUNNING a word after reload. Or, use automatic USER variables.

SIGN    ( n -- )
	Display a minus sign if n is negative. Must be used within <# and #> .

SP! ( a -- )    "s p store"
	Set the data stack pointer to address.

SP0 ( -- a )    "s p zero"
	Return the bottom address of the data stack pointer.

SP@ ( -- a )    "s p fetch"
	Return the address of the data stack pointer.

SPACE   ( -- )
	Display one space, blank character, to the current output device.

SPACES  ( +n -- )
	Display +n spaces, blank characters, to the current output device.

SPLIT  ( w -- wHI wLO)
	splits the word in TOS into two words,
	each containing one byte with leading zeroes

str ( n -- b u )
	Convert a signed integer to a numeric string.

SWAP    ( w1 w2 -- w2 w1 )
	Exchange the top two elements on the data stack.

TAP ( b b b c -- b b b' )
	Echo and store the keystroke, and update the cursor position.

temp    ( -- a )    U
	Return address of a user variable for temporary storage.

THEN    ( a -- \ -- )   I,C
	Terminate a conditional branch structure.
	Resolves a forward branch compiled by IF, ELSE, AHEAD, or AFT.

THROW   ( err# -- err# )
	Reset the state of the system to the current local error frame,
	and update the error flag.

TIB ( -- a )    "t i b"
	Address of the terminal input buffer.

TOKEN   ( -- $ ; <string> )
	Scan the current input stream for a blank delimited word.
	Move the word to the end of the names area as a packed string.
	Return the address of the packed string.

TX! ( c -- )    "t x store"
	Send a character to the output device.  Primitive of EMIT.

TYPE    ( b u -- )
	Output u characters of the string at b address to the current
	output device.

U.  ( u -- )    "u dot"
	Display the unsigned single value, use the current base.

U.R ( u +n -- ) "u dot r"
	Display the unsigned single value right-justified in a
	field of width +n, use the current base.

U<  ( u1 u2 -- t )  "u less"
	Return true if u1 is less than u2. Comparison is unsigned.

UM* ( u u -- ud )   "u m star"
	Multiply two unsigned 16-bit numbers. Return an unsigned 32-bit number.

UM+ ( u u -- ud )   "u m plus"
	Add two unsigned numbers and return a 32-bit sum.

UM/MOD  ( ud u -- ur uq )   "u m slash mod"
	Unsigned division of a 32-bit number divided by a 16-bit number.
	Return an unsigned 16-bit quotient and an unsigned 16-bit remainder.

UNTIL   ( a -- \ f -- ) I,C
	Terminate an indefinite loop structure. Condition testing is done after
	executing the code within the loop. UNTIL compiles the machine
	conditional branch instruction, and uses the address left by BEGIN
	to resolve this backward branch.

UP  ( -- a )    "u p"
	Return the address of the current user area.

-----> eliminated in v93.02 ####  USER    ( u -- ; <string> \ -- a )  D
	Build a named user variable with an offset from the current user base.
	At run-time the address of the variable is pushed on the data stack.

USOFF    ( -- a)      U
	user variable holds updated offset for UVAR

UVAR   ( -- ;  <string> )       D
	create a User VARiable with automatic offset increment
	easy to use like common Forth VARIABLEs now.
	I strongly recommend to use this instead of VARIABLE,
	because all values are automatically restored with LOADSYSTEM.

-----> eliminated in v93.02 ####   VARAD    ( -- a)      U
	user variable holds PFA of next
	defineable variable in extra RAM portion.
	necessary to make dictionary extensions
	burnable into EEPROM.

-----> eliminated in v93.02 ####   VARIABLE    ( -- ; <string> \ -- a )    D
	Build a named variable. At run-time the address of the variable is
	pushed on the data stack.

WHILE   ( a -- a a \ f -- ) I,C
	Used within an indefinite loop structure. Condition testing is done
	before executing the code within the loop. WHILE compiles the machine
	conditional branch instruction and leaves an address to be resolved
	by REPEAT .

WITHIN  ( u lo hi -- t )
	Return true if lo <= u < hi. Comparison is unsigned and circular.
	Scan the current input stream for the string delimited by 'c'.
	Return the address of the packed string.

WORDS   ( -- )
	Display the words in the CONTEXT vocabulary. Display continues until
	paused or terminated by user.

stdIO   ( -- )     "standard i o"
	Reset I/O path to SerialCom and default terminal function.

XOR ( w w -- w )
	Bitwise logical Exclusive OR.

ZLOOP   ( a -- \ -- ;R u -- [u-1] ) I,C
	Terminate a down-counting loop structure. NEXT compiles the machine
	loop instruction, pointing to the address left by FOR . The loop count
	is held on the return stack. Looping continues until the count is
	equal to zero.

[   ( -- )  I   "left bracket"
	Begin interpreting text from the input stream. C
	hange from compiling to interpreting.

[COMPILE]   ( -- ; <string> \ -- )  I   "bracket compile"
	Used only within a definition to force the compilation of the
	following IMMEDIATE word.

\   ( -- ; <string> )   I   "backslash"
	Begin a comment. The comment is terminated by the system end-of-line
	character. May be used inside or outside a definition.
	Comments are used by the eForth DOCKstation to insert DOCKstation
	directives into SourceCode transparent to the microcontroller.

]   ( -- )  "right bracket"
	Change from interpreting to compiling.

^H  ( b b b -- b b b' ; <backspace> )   "control h"
	A keyboard macro to delete characters from the current input stream.
	No action is taken if the beginning of the input stream is reached.

_TYPE   ( b u -- )  "printable type"
	Display the the string starting at the byte address b, for count u.
	Filter non-printing characters.



2. Tips zur Arbeitsweise mit eForth und FDOCK Terminalprogramm
==============================================================
( Sorry, dieser Text ist zur Zeit noch eine Baustelle )

2.1 Systeminstallation
======================
Der bliche Weg wird sein, das auf der Diskette enthaltene Binrfile in ein
EPROM zu brennen. Die Adressen sind so assembliert, da eForth den Adre-
bereich des 68HC11 von $E000 bis $FFFF einnimmt. Es kann ein EPROM des Typs
27C64 oder 27C256 verwendet werden. Wird ein EPROM des Typs 27C256 genommen,
dann kann (spter) ab $8000 eine Autostart-Applikation oder eine mit COLD
ladende Dictionary-Erweiterung gebrannt werden. Nheres dazu siehe unten.

Wenn Adrenderungen der vom Kernel alloziierten Speicherbereiche erwnscht
sind, mu der Kern neu assembliert werden. Bei anderen Assemblern als dem
von mir verwendeten UCASM ist noch ein gewisser Arbeitsaufwand zur
Anpassung der Assemblersteuerung einzuplanen.

Wo kein EPROM-Brenner verfgbar ist, kann eForth mit dem mitgelieferten
Hilfsprogramm RAMLOAD.EXE direkt mit der seriellen Schnittstelle in den
Controller geladen werden (verwendet das Hilfsprogramm LOADER.BIN) unter
Verwendung des speziell beim 68HC11 vorhandenen "Special Bootstrap Mode".
Dazu wird der Controller auch im Bereich $E000 bis $FFFF mit RAM bestckt
(an einer Variante, dort "on Board" ein EEPROM zu programmieren wird noch
getftelt). Das Programm erklrt sich selber. SEHR ratsam ist die Verwendung
eines mglichst prellfreien Reset-Tasters und eines 2-poligen Umschalters
anstelle von Jumper-Steckern zum Rckschalten in den "Expanded Multiplex"
Modus. Fehlermeldungen und Abstrze von RAMLOAD sind meistens Folge
unsauberer Reset-Impulse ! RAMLOAD startet sofort das Terminalprogramm
FDOCK.EXE, das also im gleichen Verzeichnis zu finden sein mu.
Beliebte Fehlerquellen beim Gebrauch von RAMLOAD:
--- Nicht durch Abschalten der Versorgungsspannung resetten. Nach dem
Einschalten der Versorgungsspannung "Special Bootstrap Mode" konfigurieren
und Reset-Taste drcken. Erst dann RAMLOAD starten.
--- Bei Fehlermeldung "falscher Boot-Trigger" wurde meist vergessen den
"Special Bootstrap Mode" zu konfigurieren.



2.2 eForth DOCKstation
======================
ist ein speziell zur Kommunikation mit eForth geschriebenes Terminalprogramm.
Kurzbedienunganleitung: aus dem Terminalbildschirm heraus F1 drcken.

ESC  zurck zum Programm
F1 = Kurz-Bedienungsanleitung auf Bildschirm anzeigen
F2 = Editor starten
F3 = Sourcefile zum Controller laden (und kompilieren lassen)
F4 = kompilierte Dictionary-Erweiterung und User-Daten vom Controller sichern
F5 = kompilierte Dictionary-Erweiterung und User-Daten zum Controller laden
F6 = Binrdaten vom Controller sichern
F7 = Binrdaten zum Controller laden
F8 = DOS-Funktion oder DOS-Batch-Datei aufrufen. Wenn in FDOCK.SET ein
	 Eintrag in dieser Zeile steht, wird dieser ausgefhrt. Wenn die entspr.
	 Zeile leer ist oder mit einem Leerzeichen beginnt, dann mu (bzw. kann)
	 bei jedem Aufruf neu das DOS-Kommando direkt in den nun erscheinenden
	 DOS-Bildschirm getippt werden.
F9 = Voreinstellungen ndern: Dateinamen,Pfade
F10= Voreinstellungen sichern
	 erzeugt ASCII-Datei 'FDOCK.SET' im aktuellen Verzeichnis:
	 1.Zeile:Baudrate, 2.Zeile:UART-Adr, 3.Zeile:Ser.Interrupt $B oder $C
	 (d.h. Interrupt Nr. 3 oder 4) knnen mit ASCII-Editor gendert werden.
	 danach werden Dateinamen und -Pfade gesichert.
Dateiaufbau SAVESYSTEM:RAMBOT,UPP,CP,USOFF,Binrdaten(Dict),Binrdaten(User)
	"normale" Variable werden nicht restauriert. Daher ist die Erzeugung
	von Uservariablen mit UVAR vorzuziehen, das programmiertechnische
	Verhalten ist gleich dem der "normalen" Variablen.
Dateiaufbau SAVEBIN:'EFBIN',2 Bytes Check,ADR(ASCII),LEN(ASCII),Binrdaten
	(so wird die gesicherte Datei auf Diskette gespeichert, damit
	Startadresse und Lnge automatisch beim Laden wieder angezeigt werden.
	Sollen Disk-Dateien ohne diesen Header geladen werden, wird automatisch
	die Dateilnge ermittelt,die Startadresse mu manuell eingegeben werden.)
	Bei Weiterverarbeitung der gespeicherten Daten, z.B im EPROM-Brenner,
	beachten: der Header ist derzeit 17 Bytes lang, das 18. Byte in der
	Disk-Datei ist das erste vom Controller rbergesicherte Byte.
	(Headerlnge wird bei nchster Gelegenheit auf 16 Byte gekrzt )
Dateiaufbau LOADBIN: wie SAVEBIN oder Binrdaten ohne Header. Falls in der
	Disk-Datei ein Header identifiziert wird, dient der lediglich zum auto-
	matischen Ausfllen des Requesters, er wird natrlich nicht per LOADBIN
	bertragen.


SourceLader-Direktiven: \\ ###    : folgende Source nicht an Controller
						\\ >>>    : ab hier wieder Source an Controller
									(Erklrung dazu siehe unten !)



2.3 Serielle Datenpuffer und XON/XOFF Handshake
===============================================
Sowohl der eForth Kern als auch die Dockstation sind mit Empfangs-
Datenpuffern versehen, besitzen einen im Hintergrund laufenden RX-Interrupt-
Handler und verstndigen sich gegenseitig per XON/XOFF Protokoll bei
drohendem Puffer-berlauf. XON und XOFF-Meldungen werden auf dem Terminal-
Bildschirm angezeigt. Unter normalen Bedingungen (68HC11 luft mit 8 MHZ
Quarz und Dockstation auf AT mit mindestens 12 MHZ Takt) wird die XOFF-
Bremse jedoch nie gezogen. Selbst auf einem ganz langsamen XT ist der Puffer
erst nach ca. 16 kByte SAVEBIN voll.



2.4 SourceLader Direktiven
==========================
Mit Backslash abgetrennte Kommentarzeilen werden vom eForth ignoriert,
sie bieten sich also an zur Unterbringung von Direktiven fr die
Dockstation.
Derzeit sind folgende Direktiven implementiert:
\ ###   bedeutet: Nachfolgenden SourceCode nicht an den Controller
		bertragen. Entspricht :COMMENT beim FPC
\ >>>   bedeutet: ab der nchsten Zeile wieder SourceCode zum Controller
		bertragen. Entspricht COMMENT; beim FPC
Ich habe die Direktiven absichtlich nicht COMMENT genannt, weil sie einem
viel wichtigeren Zweck dienen: der unchaotischen inkrementellen
Programmierung ! Sie ermglichen viele der Vorteile, die Blockscreens
bei der Arbeit mit Forth bieten, auch auf die Arbeit mit sequentiellen
Files zu bertragen.
Weitere Direktiven, insbesondere zum "Includen" und zum Laden von Binrfiles
aus der Source heraus, sind in Arbeit.



2.5 Inkrementierende Programmierung  / selbstdokumentierender Arbeitszyklus
===========================================================================
Angenommen, es existiert schon eine gewisse lauffhige Anfangsversion des
Programms, das jetzt erweitert werden soll.
	1. Image mit F4 sichern.
	2. Bereits lauffhigen Teil der Source mit \ ###   ... \ >>> abklammern.
	3. neue Worte mit Editor schreiben und mit F3 kompilieren.
	4. Austesten, wenn OK, mit F4 erweitertes Image sichern.
	5. Bei Absturz whrend des Testens, Reset Mikrocontroller und Image
	   mit F5 neu laden.
	6. Wenn sich frhere Worte als berholungsbedrftig erweisen: wie beim
	   Monopoly: \ >>> entsprechend hochsetzen. Falschen Teil mit FORGET
	   lschen. Neu kompilieren, testen, mit F4 sichern
	7. usw.
\ ### und \ >>> knnen mehrfach hintereinander und verschachtelt gesetzt
werden. Es wirkt immer die jeweils letzte Direktive.

Auf der Diskette ist eine Kopie des Editors SZ.COM vorhanden, das ist eine
vereinfachte PD-Version des FPC-Editors, ebenfalls aus dem Keyboard von Tom
Zimmer.
Obwohl es sicherlich komfortablere Editoren gibt, finde ich ihn ganz geeignet
fr die Programmierung von eForth, weil er auch von reinen Disketten-XT's
recht flott arbeitet und von einer Festplatte ohne groe Tricks praktisch
sofort "da" ist. Die Ausliefer-Konfiguration der Datei "FDOCK.SET" ist so
prpariert, da F8 als "Manual-Viewer" funktioniert. Es knnen auch whrend
des Programmierens eigene Kritzeleien im Manual vorgenommen werden; eine
Sicherheitskopie ist dann ratsam.
Aber dies alles ist kein MUSS, mit F9 und F10 lt sich jeder andere Editor
einbinden.



2.6 Integrierte Debugging-Tools
===============================
	SEE ist ein einfacher Debugger wie auch in anderen Forth Systemen (VIEW).

	.S gibt den gesamten momentanen Stack-Inhalt aus

	BREAK. kann beliebig in Programme eingebaut werden und gibt jedesmal
	in Form eines Breakpoints den Stack-Zustand aufs Terminal.

	Zuerst versucht BREAK. den Namen des unmittelbar zuvor ausgefhrten
	Wortes auszugeben. Falls die Dictionary-Suche erfolglos ist, wird der
	davor bearbeitete Fdellisten-Eintrag als Hexzahl ausgegeben.
	Die obersten Stack-Werte werden jeweils zuerst ausgedruckt.
	Bei der Ausgabe des Return-Stacks wird keine Prfung auf "Underflow"
	durchgefhrt. Bereits der Kaltstart legt aber schon einiges dort ab,
	so da aus einer normal laufenden Anwendung der Stack-Boden gar nicht
	erreicht werden kann. So erklren sich auch zuweilen angezeigte
	anscheinend unsinnige Eintrge des Return-Stacks.

	Das Wort hat eine Warteschleife eingebaut, die erst nach Drcken einer
	beliebigen Taste am Terminal das Programm weiterlaufen lt.



2.7 Anwendung von Forth auf Microcontrollern der unteren Leistungsklasse
========================================================================
	1. Es wird ein ziemlicher Aufwand getrieben, um zwischen User-Variablen
	und "normalen" Variablen zu unterscheiden. Dieser Aufwand ist genau
	genommen nur bei Multiuser oder Multitasking Systemen notwendig. Da
	es aber auch angesichts des Angebotes leistungsfhigerer Prozessoren
	unsinnig erscheint, umfassende Probleme auf Prozessoren wie dem
	68HC11 oder 8051 zu programmieren, stellt sich die Frage, ob es
	nicht klger ist, ein Forth auf diesen Prozessoren mglichst einfach
	und "schlank" zu halten und sich auf einen Variablentyp zu beschrnken.
	LETZE MELDUNG: bei der Implementation von BREAK. war die 8k-EPROM-Grenze
	erreicht. Daher habe ich erstmal (bis auf weiteres?) VARIABLE und USER
	geopfert. UVAR ist jetzt der einzig untersttzte Variablentyp. Der
	USER-Zeiger kann mit dem User-Wort USOFF abgefragt und verndert werden.
	So kann mit etwas Zusatzaufwand "wie konventionell" mit User-Variablen
	gearbeitet werden, z.B berflssige Bereiche berschrieben oder freier
	USER-Bereich reserviert werden. Bei der gegenwrtigen Implementation
	sind 4kByte User-Bereich installiert. Ein grerer Bereich lt sich per
	Autostart definieren (mit einiger Handarbeit allerdings).

	2. Das eForth untersttzt bisher noch keine Einrichtung neuer Vokabulare.
	Zwar sind die entsprechenden Variablen deklariert, aber nicht die zur
	Einrichtung und Verwaltung notwendigen Worte. Hier stellt sich ebenfalls
	die Frage nach den damit verbundenen Vor- und Nachteilen. Die Entwicklung
	der wichtigsten Vokabular-Worte ist aber geplant.

	3. Demgegenber finde ich es wichtig, diese "kleinen" Forth Versionen
	schneller zu machen und auch besser zu prparieren fr die Anwendung in
	verteilten Systemen, z.B. fr die Fernprogrammierung in Feldbus-Systemen
	verfgbar zu machen.



2.8 Kombination von eForth und Assemblercode
============================================
Verweis auf die Worte EXTERNAL und  HCODE   END-HCODE.
Ein einfacher Prefix Assembler ist in Arbeit.



2.9 eForth und Interrupts
=========================
eForth68 erstellt beim Kaltstart automatisch eine Interrupt-Sprungtabelle
im unteren RAM-Bereich (MOPS-Version ab $1400)
Fr jeden Interrupt sind 3 Bytes reserviert, die Interrupt-Vektoren zeigen
auf folgende Adressen:

RAMBOT .EQU $1400
	.ORG  $FFD6
	.SDW  RAMBOT    ;SCI Interrupt Vector
	.SDW  RAMBOT+3  ;SPI Transfer Complete
	.SDW  RAMBOT+6  ;Pulse Accumulator Input Edge
	.SDW  RAMBOT+9  ;Pulse Accumulator Overflow
	.SDW  RAMBOT+12 ;Timer Overflow
	.SDW  RAMBOT+15 ;Timer Output Compare 5
	.SDW  RAMBOT+18 ;Timer Output Compare 4
	.SDW  RAMBOT+21 ;Timer Output Compare 3
	.SDW  RAMBOT+24 ;Timer Output Compare 2
	.SDW  RAMBOT+27 ;Timer Output Compare 1
	.SDW  RAMBOT+30 ;Timer Input Capture 3
	.SDW  RAMBOT+33 ;Timer Input Compare 2
	.SDW  RAMBOT+36 ;Timer Input Compare 1
	.SDW  RAMBOT+39 ;Real Time Interrupt
	.SDW  RAMBOT+42 ;ext IRQ
	.SDW  RAMBOT+45 ;ext XIRQ
	.SDW  RAMBOT+48 ;SWI (Software Interrupt)
	.SDW  RAMBOT+51 ;Illegal Opcode Trap
	.SDW  RAMBOT+54 ;COP Failure
	.SDW  RAMBOT+57 ;COP Clock Monitor Fail (Reset)

Anwender knnen sich die Interrupts auf ihre Anwendung holen, wie es hier
z.B. mit dem Serial Interrupt durchgefhrt wurde:

;assemble SCI interrupt jump address to jump table
	ldaa #$7E
	staa RAMBOT     ;opcode "JMP"
	ldd #rxint
	std (RAMBOT+1)  ;destination

Interessante Mglichkeiten der Einbindung ermglicht das eForth Wort
EXTERNAL.
Formalisiertes HiLevel Interrupt Handling wird z.Zt. nicht untersttzt.



2.10 Erstellung von Autoboot-Applikationen
==========================================
Das im 68HC11 eingebaute EEPROM ermglicht sehr komfortable und differenz-
ierte Aktionen whrend des Kaltstarts einzubinden.
Um eine Blockierung bei falsch beschriebenem EEPROM auszuschlieen (und um
auch Autostart per Auenschalter an Gerten fr Servicezwecke etc abschalbar
zu machen) wird als primres Kriterium die Portleitung PE7 abgefragt. Ist
sie LOW, dann sind alle Kaltstartaktionen ABgeschaltet, ist sie HIGH, dann
sind je nach weiteren Kriterien Kaltstartaktionen EINgeschaltet. Beim MOPS
Board wird zweckmigerweise ein Widerstand 10kOhm an die VG-Leiste ST1
zwischen Pin c1 und c3 geltet. Durch einen Jumper oder Schalter nach c2
(+5V) kann die Kaltstartaktion scharf gemacht werden.
Wenn also PE7 gleich HIGH ist, dann wird folgendes in der beschriebenen
Reiehenfolge abgefragt und ausgefhrt:

1. Autocopy:
Es werden (fast) beliebig viele Speicherbereiche an andere RAM-Adressen
kopiert. Dabei sollten sich die Quell- und Zielbereich nicht berlappen,
weil keine Schutzmanahmen gegen Selbstberschreiben vorgesehen sind.

Dazu wird abgefragt: EEPROM $B602: Quell-Startadresse des 1. Speicherbereichs
					 EEPROM $B604: Ziel-Startadresse des 1. Speicherbereichs
					 EEPROM $B606: Anzahl zu kopierende Bytes

					 EEPROM $B608: Quell-Adresse des 2. Speicherbereichs
					 EEPROM $B60A: Ziel-Adresse des 2. Speicherbereichs
					 EEPROM $B60C: Anzahl zu kopierende Bytes
						.
						.
						.
					 und so weiter ...  bis: irgendein $FFFF im EEPROM
					 gelesen wird. Dann wird Autocopy abgebrochen.
					 Wenn $B602=$FFFF, dann wird gar nichts kopiert.

Mit diesem Verfahren ist es mglich, Dictionary-Erweiterungen im RAM zu er-
stellen und zu testen. Dann mit SAVEBIN sichern. Disk-Datei (abzglich
Header !) an irgendeine Stelle eines 27C256 EPROMs brennen (in dem von
$E000 bis $FFFF das eForth eingebrannt wird). Nach dem gleichen Verfahren
Interrupt-Sprungtabelle und User-Bereich ins EPROM brennen. Nun mit etwas
Handarbeit die entsprechenden Adressen und Lngen ins 68HC11-EEPROM
schreiben, damit die Bereiche beim Kaltstart an ihre ursprnglichen RAM-
Adressen zurckkopiert werden.
Weitere, spter erstellte Dictionary-Erweiterungen knnen dann in einen 3.
oder 4. EPROM-Block kopiert werden.
Kurze Erweiterungen knnen brigens direkt in das 68HC11-EEPROM kopiert
werden.
Leider habe ich noch keine Methode gefunden, kompilierten Forth-Code 100%
Adressen- und Literalsicher zu relokalisieren. Dann wrde sich nmlich
der Aufwand des Rck-Kopierens erbigen.

2.Autostart:
In die EEPROM-Zelle $B600 wird die ca der automatisch zu startenden
Applikation geschrieben:
' <APPLIKATION> B600 EEP!
Nach Verlassen dieses Wortes wird der Kaltstart normal zuende gefhrt:
die Versionsnummer ausgegeben und der Interpreter ist wie sonst auch
verfgbar. Es wre z.B. mglich, da die Applikation auf eine Terminal-
Eingabe "CONNECT" hin abbricht. Damit wre die Autostart-Applikation
per Modem fern-umprogrammierbar (batteriegepuffertes RAM oder EEPROM voraus-
gesetzt). Nach nderung "COLD" senden, dann ist wieder der Autostart-Modus
aktiv.
Wenn $B600=$FFFF gefunden wird, wird keine Applikation kalt-gestartet.
Wenn man beim Ausprobieren des Wortes EEP! allerdings irgendetwas
hineingeschrieben hat, kann es zu seltsamen Systemabstrzen kommen. Das
Gleiche gilt, wenn gebrauchte 68HC11 mit ominsem EEPROM-Inhalt ( oder
68HC11A0 mit nicht garantiertem EEPROM) benutzt werden, wenn irgendetwas am
Kernel oder an der Applikation verndert wird, und dabei vergessen wird, den
Autostart-Vektor entsprechend nachzubiegen. Zur Not erstmal PE7=LOW setzen.
